import { ButtonHTMLAttributes } from 'react';
import tw, { styled } from 'twin.macro';

export interface ButtonProps extends ButtonHTMLAttributes<HTMLButtonElement> {
  /** how important is the BUTTON */
  variant?: 'primary' | 'secondary' | 'danger';
  /** how to shade the button? */
  fill?: 'border' | 'solid' | 'text';
  /** what SIZE content to put in the button? */
  size?: 'xs' | 'sm' | 'md' | 'lg' | 'xl';
  /** what do u want the button to say */
  label?: string;
  /** should it be enabled or disabled? */
  disabled?: boolean;
  /** should the label be uppercase? */
  uppercase?: boolean;
  /** show a loading animation? */
  loading?: boolean;
  /** specify width? beware of strangeness */
  width?: 'xs' | 'sm' | 'md' | 'lg' | 'xl' | '2xl' | 'full' | 'auto';
  /** specify height? beware of strangeness  */
  height?: 'xs' | 'sm' | 'md' | 'lg' | 'xl' | '2xl' | 'full' | 'auto';
  /** want to remove default margin? */
  removeMargin?: boolean;
  /** on a click, do smth */
  onClick?: (event: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
}

const sizeStyles = {
  xs: tw`m-0.5 px-1 py-0.5 text-xs`,
  sm: tw`m-1 px-2 py-1 text-sm`,
  md: tw`m-2 px-3 py-1.5 text-base`,
  lg: tw`m-3 px-4 py-2 text-lg`,
  xl: tw`m-4 px-5 py-2.5 text-xl`,
};

const widthStyles = {
  xs: tw`w-16`,
  sm: tw`w-20`,
  md: tw`w-28`,
  lg: tw`w-32`,
  xl: tw`w-40`,
  '2xl': tw`w-48`,
  '3xl': tw`w-56`,
  full: tw`w-full`,
  auto: tw`w-auto`,
};

const heightStyles = {
  xs: tw`h-8`,
  sm: tw`h-10`,
  md: tw`h-12`,
  lg: tw`h-14`,
  xl: tw`h-16`,
  '2xl': tw`h-20`,
  '3xl': tw`h-24`,
  full: tw`h-full`,
  auto: tw`h-auto`,
};

const textAnimationTW = `relative after:(absolute h-px w-0 bottom-0 duration-200 left-1/2) canhover:hover:after:(w-full left-0)`;

const solidCommon = 'shadow-lg leading-relaxed';
const borderCommon = 'bg-transparent border-2 leading-relaxed';
const textCommon = 'bg-transparent leading-relaxed';

const variantStyles = {
  primary: {
    solid: tw`${solidCommon}
      text-white bg-blue-400
      canhover:hover:enabled:(bg-blue-500 active:bg-blue-600) 
      nohover:active:bg-blue-600`,
    border: tw`${borderCommon} 
      text-blue-400 border-blue-400 
      canhover:hover:enabled:(text-white bg-blue-400 active:bg-blue-600)
      nohover:active:bg-blue-100
      disabled:(text-blue-300 border-blue-400/40)`,
    text: tw`${textCommon} ${textAnimationTW}
      text-blue-400
      enabled:after:bg-blue-400
      disabled:text-blue-300
      active:text-blue-600
      enabled:active:after:bg-blue-600`,
  },
  secondary: {
    solid: tw`${solidCommon}
      text-white bg-gray-400
      canhover:hover:enabled:(bg-gray-500 active:bg-gray-600) 
      nohover:active:bg-gray-600`,
    border: tw`${borderCommon} 
      text-gray-400 border-gray-400 
      canhover:hover:enabled:(text-white bg-gray-400 active:bg-gray-600)
      nohover:active:bg-gray-100
      disabled:(text-gray-300 border-gray-400/40)`,
    text: tw`${textCommon} ${textAnimationTW}
      text-gray-400
      enabled:after:bg-gray-400
      disabled:text-gray-300
      active:text-gray-600
      enabled:active:after:bg-gray-600`,
  },
  danger: {
    solid: tw`${solidCommon}
      text-white bg-red-400
      canhover:hover:enabled:(bg-red-500 active:bg-red-600) 
      nohover:active:bg-red-600`,
    border: tw`${borderCommon} 
      text-red-400 border-red-400 
      canhover:hover:enabled:(text-white bg-red-400 active:bg-red-600)
      nohover:active:bg-red-100
      disabled:(text-red-300 border-red-400/40)`,
    text: tw`${textCommon} ${textAnimationTW}
      text-red-400
      enabled:after:bg-red-400
      disabled:text-red-300
      active:text-red-600
      enabled:active:after:bg-red-600`,
  },
};

// there must be, a better wayyyyyyyyyy to show a loading animation
// doing it here b/c i don't want the hover/active to interfere while loading
const loadingCommon =
  'text-transparent	flex items-center justify-center before:(absolute h-4 w-4 border-b-2 rounded-full animate-spin)';

const loadingStyles = {
  primary: {
    solid: tw`${solidCommon} ${loadingCommon} bg-blue-400`,
    border: tw`${borderCommon} ${loadingCommon} border-blue-400 before:border-blue-400
    disabled:border-blue-400/60 disabled:before:border-blue-400/60`,
    text: tw`${textCommon} ${loadingCommon}
    before:border-blue-400
    disabled:before:border-blue-400/60`,
  },
  secondary: {
    solid: tw`${solidCommon} ${loadingCommon} bg-gray-400`,
    border: tw`${borderCommon} ${loadingCommon} border-gray-400 before:border-gray-400
    disabled:border-gray-400/60 disabled:before:border-gray-400/60`,
    text: tw`${textCommon} ${loadingCommon}
    before:border-gray-400
    disabled:before:border-gray-400/60`,
  },
  danger: {
    solid: tw`${solidCommon} ${loadingCommon} bg-red-400`,
    border: tw`${borderCommon} ${loadingCommon} border-red-400 before:border-red-400
    disabled:border-red-400/60 disabled:before:border-red-400/60`,
    text: tw`${textCommon} ${loadingCommon}
    before:border-red-400
    disabled:before:border-red-400/60`,
  },
};

const StyledButton = styled.button(
  ({
    variant = 'primary',
    fill = 'solid',
    size = 'md',
    width = 'auto',
    height = 'auto',
    disabled = false,
    uppercase = false,
    loading = false,
    removeMargin = false,
  }: ButtonProps) => [
    // common
    tw`rounded-sm border-0 font-normal cursor-pointer inline-block 
      duration-300 ease-in-out break-all overflow-ellipsis overflow-hidden`,

    // sizes
    sizeStyles[size],

    // remove margin
    removeMargin && tw`m-0`,

    // btn width
    widthStyles[width],

    // btn height
    heightStyles[height],

    // variant n fill
    !loading && variantStyles[variant][fill],

    // loading
    loading && loadingStyles[variant][fill],

    // common disabled style
    disabled && tw`bg-opacity-40 cursor-not-allowed`,

    // case
    uppercase && tw`uppercase`,
  ]
);

StyledButton.defaultProps = {
  variant: 'primary',
  fill: 'solid',
  size: 'md',
  width: 'auto',
  height: 'auto',
  disabled: false,
  uppercase: false,
  loading: false,
  type: 'button',
};

const Button = (props: ButtonProps) => (
  <StyledButton {...props}>{props.label}</StyledButton>
);

export default Button;
